@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 4044 2008-07-18 12:45:13Z wwalker $"
-__version__   = "$Revision: 4044 $"
-__date__      = "$Date: 2008-07-18 08:45:13 -0400 (Fri, 18 Jul 2008) $"
+__id__        = "$Id: braillegenerator.py 4072 2008-08-01 20:47:02Z wwalker $"
+__version__   = "$Revision: 4072 $"
+__date__      = "$Date: 2008-08-01 16:47:02 -0400 (Fri, 01 Aug 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -80,6 +80,8 @@ class BrailleGenerator:
              self._getBrailleRegionsForDialog
         self.brailleGenerators[pyatspi.ROLE_DIRECTORY_PANE]      = \
              self._getBrailleRegionsForDirectoryPane
+        self.brailleGenerators[pyatspi.ROLE_EMBEDDED]            = \
+             self._getBrailleRegionsForEmbedded
         self.brailleGenerators[pyatspi.ROLE_FRAME]               = \
              self._getBrailleRegionsForFrame
         self.brailleGenerators[pyatspi.ROLE_HTML_CONTAINER]      = \
@@ -548,6 +550,39 @@ class BrailleGenerator:
 
         return self._getDefaultBrailleRegions(obj)
 
+    def _getBrailleRegionsForEmbedded(self, obj, role=None):
+        """Gets text to be displayed for the current embedded object.
+
+        Arguments:
+        - obj: an Accessible
+        - role: Role to use as an override.
+
+        Returns a list where the first element is a list of Regions to
+        display and the second element is the Region which should get
+        focus.
+        """
+
+        self._debugGenerator("_getBrailleRegionsForEmbedded", obj)
+
+        regions = []
+
+        text = ""
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
+        if not text:
+            try:
+                text = obj.getApplication().name
+            except:
+                pass
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
+
     def _getBrailleRegionsForFrame(self, obj):
         """Get the braille for a frame.
 
@@ -864,7 +899,7 @@ class BrailleGenerator:
             regions.append(braille.Region(" " + text))
 
         if settings.presentReadOnlyText \
-           and not obj.getState().contains(pyatspi.STATE_EDITABLE):
+           and self._script.isReadOnlyTextArea(obj):
             regions.append(braille.Region(" " \
                                           + settings.brailleReadOnlyString))
 
