@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: script.py 3996 2008-06-23 16:25:15Z richb $"
-__version__   = "$Revision: 3996 $"
-__date__      = "$Date: 2008-06-23 12:25:15 -0400 (Mon, 23 Jun 2008) $"
+__id__        = "$Id: script.py 4071 2008-07-30 16:56:57Z joanied $"
+__version__   = "$Revision: 4071 $"
+__date__      = "$Date: 2008-07-30 12:56:57 -0400 (Wed, 30 Jul 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -331,6 +331,17 @@ class Script(default.Script):
         except:
             debug.printException(debug.LEVEL_WARNING)
 
+    def isReadOnlyTextArea(self, obj):
+        """Returns True if obj is a text entry area that is read only."""
+        state = obj.getState()
+        readOnly = obj.getRole() == pyatspi.ROLE_TEXT \
+                   and state.contains(pyatspi.STATE_FOCUSABLE) \
+                   and not state.contains(pyatspi.STATE_EDITABLE)
+        debug.println(debug.LEVEL_ALL,
+                      "soffice.script.py:isReadOnlyTextArea=%s for %s" \
+                      % (readOnly, debug.getAccessibleDetails(obj)))
+        return readOnly
+
     def adjustForWriterTable(self, obj):
         """Check to see if we are in Writer, where the object with focus
         is a paragraph, and the parent is the table cell. If it is, then,
@@ -1747,11 +1758,15 @@ class Script(default.Script):
             self.lastCell = event.source.parent
             return
 
-        # Two events are received when the caret moves
-        # to a new paragraph in oowriter. The first is a focus event
-        # (in the form of object:state-changed:focused
-        # instead of focus:). The second is a caret-moved
-        # event. Just set the locusOfFocus for the first event.
+        # When a new paragraph receives focus, we get a caret-moved event and
+        # two focus events (the first being object:state-changed:focused).
+        # The caret-moved event will cause us to present the text at the new
+        # location, so it is safe to set the locusOfFocus silently here.
+        # However, if we just created a new paragraph by pressing Return at
+        # the end of the current paragraph, we will only get a caret-moved
+        # event for the paragraph that just gave up focus (detail1 == -1).
+        # In this case, we will keep displaying the previous line of text,
+        # so we'll do an updateBraille() just in case.
         #
         if event.type.startswith("object:state-changed:focused"):
             rolesList = [pyatspi.ROLE_PARAGRAPH, \
@@ -1760,10 +1775,12 @@ class Script(default.Script):
                          pyatspi.ROLE_PANEL, \
                          pyatspi.ROLE_ROOT_PANE, \
                          pyatspi.ROLE_FRAME]
-            if self.isDesiredFocusedItem(event.source, rolesList) and \
-               event.source != self.currentParagraph:
+            if self.isDesiredFocusedItem(event.source, rolesList) \
+               and event.source != self.currentParagraph \
+               and event.detail1 == 1:
                 self.currentParagraph = event.source
                 orca.setLocusOfFocus(event, event.source, False)
+                self.updateBraille(event.source)
                 return
 
         # If we are in the sbase Table Wizard, try to reduce the numerous
