@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 4044 2008-07-18 12:45:13Z wwalker $"
-__version__   = "$Revision: 4044 $"
-__date__      = "$Date: 2008-07-18 08:45:13 -0400 (Fri, 18 Jul 2008) $"
+__id__        = "$Id: speechgenerator.py 4072 2008-08-01 20:47:02Z wwalker $"
+__version__   = "$Revision: 4072 $"
+__date__      = "$Date: 2008-08-01 16:47:02 -0400 (Fri, 01 Aug 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -78,6 +78,8 @@ class SpeechGenerator:
              self._getSpeechForDialog
         self.speechGenerators[pyatspi.ROLE_DIRECTORY_PANE]      = \
              self._getSpeechForDirectoryPane
+        self.speechGenerators[pyatspi.ROLE_EMBEDDED]            = \
+             self._getSpeechForEmbedded
         self.speechGenerators[pyatspi.ROLE_FRAME]               = \
              self._getSpeechForFrame
         self.speechGenerators[pyatspi.ROLE_HTML_CONTAINER]      = \
@@ -616,6 +618,39 @@ class SpeechGenerator:
 
         return utterances
 
+    def _getSpeechForEmbedded(self, obj, already_focused, role=None):
+        """Gets a list of utterances to be spoken for the current
+        embedded component (i.e., something in a panel).
+
+        Arguments:
+        - obj: an Accessible
+        - already_focused: False if object just received focus
+        - role: A role that should be used instead of the Accessible's 
+          possible role.
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = []
+
+        label = self._getSpeechForObjectLabel(obj)
+        utterances.extend(label)
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
+        if not utterances:
+            try:
+                utterances.append(obj.getApplication().name)
+            except:
+                pass
+
+        self._debugGenerator("_getSpeechForEmbedded",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        return utterances
+
     def _getSpeechForFrame(self, obj, already_focused):
         """Get the speech for a frame.
 
@@ -962,7 +997,7 @@ class SpeechGenerator:
                 utterances.append(obj.name)
 
         if settings.presentReadOnlyText \
-           and not obj.getState().contains(pyatspi.STATE_EDITABLE):
+           and self._script.isReadOnlyTextArea(obj):
             utterances.append(settings.speechReadOnlyString)
 
         if obj.getRole() != pyatspi.ROLE_PARAGRAPH:
